# Movable Type plugin for generate and display thumbnail image.
#
# Release 1.00 (Aug 01, 2006) initial version.
# Release 1.01 (Aug 03, 2006) change thumbnail filename and add 'permalink' attribute value.
# Release 1.02 (Aug 18, 2006) add error check.
# Release 1.10 (Aug 23, 2006) change the method of getting the image.
# Release 1.11 (Aug 24, 2006) change method to 'BlobToImage' from 'Read'.
# Release 1.12 (Aug 26, 2006) change method to 'ImageToBlob' from 'Write'.
#
# This software is provided as-is. You may use it for commercial or 
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2004-2006 Yujiro Araki

package MT::Plugin::Thumbnail;
use strict;
use MT;
use File::Spec;
use base qw(MT::Plugin);

our $VERSION = '1.12';
our $overwrite = '0';

my $plugin = MT::Plugin::Thumbnail->new({
    name => 'Thumbnail',
    description => 'Generate thumbnail and img tag.',
    doc_link => 'http://www.koikikukan.com/archives/2006/08/01-000101.php',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    version => $VERSION,
    blog_config_template => 'config.tmpl',
    settings => new MT::PluginSettings([
        ['def_url', { Default => 'http://' }],
        ['def_path'],
        ['width'], ['height'],
        ['Index', { Default => 1 }],
        ['Category', { Default => 0 }],
        ['Monthly', { Default => 0 }],
        ['Weekly', { Default => 0 }],
        ['Daily', { Default => 0 }],
        ['Individual', { Default => 0 }],
    ]),
});
MT->add_plugin($plugin);

MT::Template::Context->add_tag(Thumbnail => \&thumbnail);
use MT::Template::Context;

sub thumbnail {
    my $ctx = shift;
    my $args = shift;
    eval { require Image::Magick };
    return $ctx->error(MT->translate("Can't load Image::Magick: [_1]", $@)) if $@;

    my $blog_id = $ctx->stash('blog_id');
    my $config = config('blog:' . $blog_id);

    my $path = $config->{def_path};
    return $ctx->error("No setting default path.") if ($path eq '');
    chop $path if $path =~ m!.*/$!;

    my $url = $config->{def_url};
    return $ctx->error("No setting default URL.") if ($url eq 'http://');
    $url .= '/' unless $url =~ m!/$!;

    # get image
    my $file = (defined($args->{file}) ? $args->{file} : '');
    my $fmgr = MT::Blog->load($blog_id, {cached_ok=>1})->file_mgr;
    my $i_file = File::Spec->catfile($path, $file);
    my $data = $fmgr->get_data($i_file, 'upload')
                 or return $ctx->error(MT->translate("Reading '[_1]' failed: [_2]", $i_file, $fmgr->errstr));

    # new image
    (my $ext = $file) =~ s/.*\.(.*)/$1/;
    my $img = Image::Magick->new(magick => lc($ext));

    return $ctx->error(MT->translate("Can't load image: [_1]", "$path/$file")) if $img->BlobToImage($data);
    my ($width, $height) = $img->Get('width', 'height');
    return $ctx->error(MT->translate("Illegal size of image: width:[_1] height:[_2]", $width, $height)) if (!$width || !$height);

    my $archive_type = $ctx->{current_archive_type};
    $archive_type = 'Index' if ($archive_type eq '');
    my $class = defined($args->{class}) ?
                (' class="' . $args->{class}.' '.lc($archive_type).'_image'.'"') :
                (' class="'.lc($archive_type).'_image'.'"');

    # no thumbnail
    my $alt = (defined($args->{alt}) ? $args->{alt} : '');
    return qq{<img src="$url$file" width="$width" height="$height" title="$alt" alt="$alt"$class />} unless ($config->{$archive_type});

    my $thmb_width = (defined($args->{width}) ? $args->{width} : '');
    my $thmb_height = (defined($args->{height}) ? $args->{height} : '');

    if ($thmb_width && !$thmb_height) {
        $thmb_height = int(($height * ($thmb_width/$width)) + 0.5);
    } elsif (!$thmb_width && $thmb_height) {
        $thmb_width = int(($width * ($thmb_height/$height)) + 0.5);
    } elsif (!$thmb_width && !$thmb_height) {
        $thmb_width = $config->{width};
        $thmb_height = $config->{height};
    }
    return $ctx->error(MT->translate("No setting thumbnail size.")) if (!$thmb_width || !$thmb_height);

    # thumbnail
    (my $thmb = $file) =~ s/(.*)\.(\w+)$/$1-thumb.$2/;
    if ((! -e "$path/$thmb") || $overwrite) {
        my $err = $img->can('Resize') ?
              $img->Resize(width => $thmb_width, height => $thmb_height, blur=>0.7) :
              $img->Scale(width => $thmb_width, height => $thmb_height);
         return $ctx->error(MT->translate("Scaling to [_1]x[_2] failed: [_3]", $thmb_width, $thmb_height, $err)) if $err;
    $img->Profile("*") if $img->can('Profile');
    $fmgr->put_data($img->ImageToBlob, "$path/$thmb", 'upload')
        or return $ctx->error(MT->translate("Error writing to '[_1]': [_2]", "$path/$thmb", $fmgr->errstr));

    }

    $class = defined($args->{class}) ?
             (' class="' . $args->{class}.' '.lc($archive_type).'_thumb'.'"') :
             (' class="'.lc($archive_type).'_thumb'.'"');
    my $tag = qq{<img src="$url$thmb" width="$thmb_width" height="$thmb_height" title="$alt" alt="$alt"$class />};
    my $link = $args->{link};
    if ($link) {
        if ($link eq 'image') {
            return qq{<a href="$url$file">$tag</a>};
        } elsif($link eq 'lightbox'){
            return qq{<a href="$url$file" rel="lightbox">$tag</a>};
        } elsif($link eq 'permalink'){
            if ($archive_type eq 'Individual') {
                return qq{$tag};
            } else {
                my $permalink = $ctx->stash('entry')->permalink;
                return qq{<a href="$permalink">$tag</a>};
            }
        } else {
            return qq{<a href="$link">$tag</a>};
        }
    } else {
        return $tag;
    }
}

sub config {
    my $config = {};
    return $config unless $plugin;
    require MT::Request;
    my $scope = shift || 'system';
    $config = MT::Request->instance->cache('thumbnail_config_'.$scope);
    if (!$config) {
        $config = $plugin->get_config_hash($scope);
        MT::Request->instance->cache('thumbnail_config_'.$scope, $config);
    }
    $config;
}

1;
